@@ -139,10 +139,13 @@ DlgAddCity::updateLocations( const QHash<QString, QVariant> & container )
 					{
 						QString sStationType, sDistance, sItemText;
 						getExtraLocationData( container.value(sExtra), sStationType, sDistance );
-						if( !sStationType.isEmpty() )
+						
+						if (sStationType.compare("airport", Qt::CaseInsensitive) == 0)
 							sItemText += i18n(sStationType.toUtf8().constData()) + ": ";
+						
 						sItemText += sLocation;
-						if( !sDistance.isEmpty() )
+						
+						if (!sDistance.isEmpty())
 							sItemText += QString(" (%1 %2)").arg(i18n("distance:")).arg(sDistance);
 						
 						item->setText(sItemText);
