@@ -116,21 +116,7 @@ DesktopPainter::drawPage(QPainter * painter, const QRect & contentsRect) const
 		
 		case Yawp::SatellitePage:
 			if (stateMachine()->hasPage(Yawp::SatellitePage))
-			{
-				qreal dScale = contentsRect.width() / YAWP_ORIG_SIZEX;
-				const CityWeather * city = stateMachine()->currentCity();
-				
-				QRect satelliteRect(
-					contentsRect.left(),
-					qRound(contentsRect.top() + 55.0f * dScale),
-					contentsRect.width(),
-					qRound(185.0f * dScale) );
-
-				painter->drawImage(
-				      satelliteRect,
-				      city->satelliteImage(),
-				      QRect(QPoint(0, 0), city->satelliteImage().size()) );
-			}
+				drawSatelliteImage(painter, contentsRect);
 		break;
 		
 		default:
@@ -414,8 +400,8 @@ DesktopPainter::drawForecast(QPainter * painter, const QRect & detailsContentsRe
 		{
 			rect.setTop( detailsContentsRect.top() + 3.0f * dScale);
 			rect.setHeight( detailsContentsRect.height() * 0.95f );
-			rect.setLeft(rect.left() - 3.0f * dScale);
-			rect.setRight(rect.right() + 3.0f * dScale);
+			rect.setLeft(rect.left() - 5.0f * dScale);
+			rect.setRight(rect.right() + 5.0f * dScale);
 			
 			drawFancyTemperature(painter, stateMachine()->day(dayIndex), rect);
 		}
@@ -428,8 +414,8 @@ DesktopPainter::drawForecast(QPainter * painter, const QRect & detailsContentsRe
 		  
 			rect.moveTop(rect.bottom() + rect.height() * 0.14f);
 			rect.setHeight(rect.height() * 0.8f);
-			rect.setLeft(rect.left() - 3.0f * dScale);
-			rect.setRight(rect.right() + 3.0f * dScale);
+			rect.setLeft(rect.left() - 8.0f * dScale);
+			rect.setRight(rect.right() + 8.0f * dScale);
 
 			drawTemperature(painter, weather, rect);
 		}
@@ -537,8 +523,8 @@ DesktopPainter::drawDetails(QPainter * painter, int dayIndex, const QRect & deta
 		
 
 		QString creditText = city->credit();
-		if( !city->creditURL().isEmpty() && fm.width(city->creditURL()) < creditRect.width() )
-			creditText += "\n" + city->creditURL();
+//		if( !city->creditURL().isEmpty() && fm.width(city->creditURL()) < creditRect.width() )
+//			creditText += "\n" + city->creditURL();
 
 		drawText( painter, creditRect, Qt::AlignVCenter | Qt::AlignHCenter | Qt::TextWordWrap, creditText );
 		
@@ -591,6 +577,50 @@ DesktopPainter::drawDetails(QPainter * painter, int dayIndex, const QRect & deta
 	dEndFunct();
 }
 
+void
+DesktopPainter::drawSatelliteImage(QPainter * painter, const QRect & contentsRect) const
+{
+	const float dScale = contentsRect.width() / YAWP_ORIG_SIZEX;
+	const CityWeather * city = stateMachine()->currentCity();
+	
+	int offset = 55.0f * dScale;
+	QRect satelliteRect(
+		contentsRect.left(),
+		qRound(contentsRect.top() + offset),
+		contentsRect.width(),
+		qRound(contentsRect.height() - offset) );
+	
+	// we draw a black background to hide the details header rectangle,
+	// that contains the daynames in preview mode and "Forecast details" in detail mode
+	painter->save();
+	painter->setBrush(QColor(Qt::black));
+	painter->setPen(QColor(Qt::black));
+	painter->drawRect(satelliteRect);
+	painter->restore();
+	
+	QImage satelliteImage = city->satelliteImage();
+	
+	/** Try to scale the satellite image properly
+	 */
+	const float imageAspectRatio = ((float)satelliteImage.width()) / ((float)satelliteImage.height());
+	int heightForMaxWidth = satelliteRect.width() / imageAspectRatio;
+	int widthForMaxHeight = satelliteRect.height() * imageAspectRatio;
+	
+	if (heightForMaxWidth <= satelliteRect.height())
+	{
+		int heightOffset = qRound( ((float)(satelliteRect.height() - heightForMaxWidth)) / 2.0f );
+		satelliteRect.adjust(0, heightOffset, 0, -heightOffset);
+	}
+	else
+	{
+		int widthOffset = qRound( ((float)(satelliteRect.width() - widthForMaxHeight)) / 2.0f );
+		satelliteRect.adjust(widthOffset, 0, -widthOffset, 0);
+	}
+	
+	//--- Draw the satellite well sized ---
+	painter->drawImage( satelliteRect, satelliteImage, QRect(QPoint(0, 0), satelliteImage.size()) );
+}
+
 void
 DesktopPainter::drawTemperature(QPainter * painter, const YawpWeather * weather, const QRect & tempRect) const
 {
@@ -966,7 +996,7 @@ DesktopPainter::getDetailsContentsRect(const QRect & contentsRect) const
 	return QRect( contentsRect.left(),
 	              qRound(contentsRect.top() + 150.0f * dScale),
 	              contentsRect.width(),
-	              qRound(103.0f * dScale) );
+	              qRound(102.0f * dScale) );
 }
 
 void
