@@ -178,7 +178,23 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 
 	//--- Get weather forecast information ---
 	cityInfo.setCredit( data.value("Credit").toString() );
-	cityInfo.setCreditURL( data.value("Credit Url").toString() );
+	
+	//-- This is a stupid workaround ---
+	// ION BBC Weather Engine is returning the url to the weather forecast of current city
+	// Our own ION's accuweather and wunderground is following this example since version 0.4.0
+	// ION "Environment Canada" and "NOAA" does not returning "Credit Url" at all, probably because the provider
+	// does not support the information to send the url to the weather forecast
+	// Our own ION Google is following the example of ION "Environment Canada" and "NOAA"
+	// But ION wetter.com is returning the base url to the weather provider.
+	// Up to KDE version 4.2 all ION's was returning the base url to the weather provider,
+	// but at one point this has been changed and probably KDE-Developer forgot about ION wetter.com
+	//... at least I think so...
+	// Therefore we just check whether Credit Url contains a path or not - in the second case we suppose
+	// it is just the base url to the weather provider. In this case we do not use the credit Url.
+	KUrl creditUrl(data.value("Credit Url").toString());
+	if (creditUrl.isValid() && creditUrl.hasPath())
+		cityInfo.setCreditUrl(data.value("Credit Url").toString());
+	
 	int iForecastDays = data.value("Total Weather Days", "0").toInt();
 	QDate currDate;	// date of current day
 	int iTodayDays(0);
@@ -1031,7 +1047,7 @@ WeatherDataProcessor::Private::createPropertyText( const Yawp::DetailsProperty p
 	case Yawp::UV:
 		if( weather.uvIndex() < SHRT_MAX )
 		{
-			return i18n("UV Index: %1  %2",
+			return i18n("UV Index: %1 %2",
 				weather.uvIndex(), weather.uvRating());
 		}
 		break;
