@@ -99,7 +99,7 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 	WeatherDataProcessor * pDataProcessor = new WeatherDataProcessor( &m_storage );
 	m_pWeatherModel = new WeatherServiceModel( &m_storage, this, pDataProcessor );
 	m_pWeatherModel->setObjectName("EngineModel");
-	
+
 	m_stateMachine.setServiceModel( m_pWeatherModel );
 
 	//--- Set Default Configuration Values ---
@@ -150,7 +150,7 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 		#endif
 	#endif
 	}
-	
+
 	m_configData.todaysWeatherPanelFormat	= Yawp::PanelTemperature | Yawp::PanelIcon;
 	m_configData.forecastWeatherPanelFormat	= Yawp::PanelTemperature | Yawp::PanelIcon;
 	m_configData.iPanelForecastDays = 3;
@@ -167,7 +167,7 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 	m_configData.bUseCompactPanelLayout = false;
 	m_configData.bUseExtendedTooltip = true;
 	m_configData.extendedTooltipOptions = Yawp::PreviewPage | Yawp::SatellitePage;
-	
+
 	m_configData.bTraverseLocationsPeriodically = false;
 	m_configData.iTraverseLocationTimeout = 30;
 
@@ -179,9 +179,9 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 		<< Yawp::Visibility
 		<< Yawp::Dewpoint
 		<< Yawp::UV;
-		
+
 	// Contextual actions - thanks to Ezequiel, ported from stdin plasmoid :-)
-	m_pManualUpdate = new QAction ( i18n ("&Refresh"), this );
+	m_pManualUpdate = new QAction( i18n ("&Refresh"), this );
 	m_pManualUpdate->setIcon ( KIcon ( "view-refresh" ) );
 	addAction( "refresh", m_pManualUpdate );
 	connect ( m_pManualUpdate, SIGNAL(triggered()), m_pWeatherModel, SLOT(reconnectEngine()) );
@@ -199,6 +199,12 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 	m_pGrpActionCities = new QActionGroup( this );
 	m_pGrpActionCities->setExclusive( true );
 	connect( m_pGrpActionCities, SIGNAL(triggered(QAction *)), this, SLOT(changeCity(QAction *)) );
+	
+	m_pOpenForecastUrl = new QAction( i18n("Open Forecast URL"), this);
+	m_pOpenForecastUrl->setIcon( KIcon("text-html") );
+	//addAction( "open_url", m_pOpenForecastUrl);
+	connect( m_pOpenForecastUrl, SIGNAL(triggered()), this, SLOT(openForecastUrl()) );
+	m_pOpenForecastUrl->setEnabled(false);
 
 	QAction * separator2 = new QAction ( this );
 	separator2->setSeparator ( true );
@@ -207,8 +213,9 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 	m_actions.append( aboutAction );
 	m_actions.append( separator1 );
 	m_actions.append( m_pCitySubMenu );
+	m_actions.append( m_pOpenForecastUrl );
 	m_actions.append( separator2 );
-	
+
 	/*** About data ***/
 	m_aboutData = new KAboutData ( "plasma_yawp",
 		QByteArray (), ki18n( YAWP_NAME ), YAWP_VERSION_STRING,
@@ -287,6 +294,8 @@ YaWP::YaWP( QObject * parent, const QVariantList & args )
 	setAspectRatioMode(Plasma::KeepAspectRatio);
 	setPassivePopup(true);
 	
+	resize(273, 255);	// set a fixed initial size that fits the desktop mode. We simply suppose we are always in desktop-mode.
+
 	dEndFunct();
 }
 
@@ -294,9 +303,9 @@ YaWP::~YaWP()
 {
 	dStartFunct();
 	Plasma::ToolTipManager::self()->clearContent(this);
-	
+
 	destroyExtenderItem();
-	
+
 	if (hasFailedToLaunch())
 	{
 		// Do some cleanup here
@@ -306,13 +315,13 @@ YaWP::~YaWP()
 		// Save settings
 		saveConfig();
 	}
-	
+
 	if (m_pAppletPainter != 0)
 		delete m_pAppletPainter;
-	
+
 	delete m_pWeatherModel;
 	delete m_aboutData;
-	
+
 	dEndFunct();
 }
 
@@ -326,7 +335,6 @@ YaWP::init()
 	loadConfig();
 	loadCustomTheme();
 	initAppletPainter();
-	resize( m_pAppletPainter->getSize(QSize()) );
 
 	//--- just 4 debug ---
 /*	CityWeather dummyCity1;
@@ -347,34 +355,31 @@ YaWP::init()
 	dummyCity2.setTimeZone("Asia/Shanghai");
 	m_pWeatherModel->addCity(dummyCity2);
 	//--- end of debug section ---
-*/	
+*/
 	updateCitySubMenu();
 	setCityIndex( m_configData.iCityIndex );
 
 	const bool bHasCities = (m_pWeatherModel->rowCount() > 0);
-	m_pManualUpdate->setEnabled( bHasCities );
-	m_pCitySubMenu->setEnabled( bHasCities);
-	
+
 	if (bHasCities)
 		m_iIdPendingEngineConnection = startTimer( m_configData.iStartDelay*60*1000 );
 	else
-		QTimer::singleShot(0, this, SLOT(slotConfigurationRequired()) );
-	
+		setConfigurationRequired( true );
+
+	m_pManualUpdate->setEnabled( bHasCities );
+	m_pCitySubMenu->setEnabled( bHasCities);
+
 	startTraverseLocationTimeout();
-	
+
 	// Every 5 days, 15 minutes after start of applet, cache directory will be checked to remove old files.
 	if (QDate::currentDate().dayOfYear() % 5 == 0)
 		QTimer::singleShot(15*60*1000, this, SLOT(slotStartCacheCleanUp()) );
+
+	PopupApplet::init();
 	
 	dEndFunct();
 }
 
-void
-YaWP::slotConfigurationRequired()
-{
-	setConfigurationRequired( true );
-}
-
 void
 YaWP::loadConfig()
 {
@@ -491,7 +496,7 @@ YaWP::loadConfig()
 			cityInfo.setCountry(     QUrl::fromPercentEncoding(lst.at(2).toUtf8()) );
 			cityInfo.setCountryCode( QUrl::fromPercentEncoding(lst.at(3).toUtf8()) );
 			cityInfo.setExtraData(   QUrl::fromPercentEncoding(lst.at(4).toUtf8()) );
-			
+
 			if( lst.count() > 5 )
 				cityInfo.setTimeZone( QUrl::fromPercentEncoding(lst.at(5).toUtf8()) );
 			if( !cityInfo.timeZone().isValid() )
@@ -502,10 +507,10 @@ YaWP::loadConfig()
 			}
 			if( cityInfo.countryCode().isEmpty() && cityInfo.timeZone().isValid() )
 				cityInfo.setCountryCode( cityInfo.timeZone().countryCode() );
-			
+
 			dDebug() << cityInfo.provider() << cityInfo.city() << cityInfo.country() << cityInfo.countryCode()
 			         << cityInfo.extraData() << cityInfo.timeZone().name() << cityInfo.timeZone().countryCode();
-			
+
 			m_pWeatherModel->addCity( cityInfo );
 
 			iCityIndex += 1;
@@ -528,7 +533,7 @@ YaWP::saveConfig()
 	cfg.writeEntry( "start delay",				(int)m_configData.iStartDelay );
 	cfg.writeEntry( "traverse locations",		m_configData.bTraverseLocationsPeriodically );
 	cfg.writeEntry( "traverse locations timeout",	m_configData.iTraverseLocationTimeout );
-	
+
 	cfg.writeEntry( "system.distance",          (int)m_configData.distanceSystem );
 	cfg.writeEntry( "system.pressure",          (int)m_configData.pressureSystem );
 	cfg.writeEntry( "system.temperature",		(int)m_configData.temperatureSystem );
@@ -615,11 +620,11 @@ void
 YaWP::constraintsEvent(Plasma::Constraints constraints)
 {
 	dStartFunct();
-	
+
 	if (constraints.testFlag(Plasma::FormFactorConstraint))
 	{
 		initAppletPainter();
-		
+
 		if (m_pAppletPainter->formFactor() == Plasma::Planar)
 		{
 			if (m_configData.sBackgroundName == QLatin1String("default") &&
@@ -632,7 +637,7 @@ YaWP::constraintsEvent(Plasma::Constraints constraints)
 			{
 				setBackgroundHints(NoBackground);
 			}
-			
+
 		}
 		else if (m_pAppletPainter->formFactor() == Plasma::Horizontal)
 		{
@@ -644,7 +649,7 @@ YaWP::constraintsEvent(Plasma::Constraints constraints)
 			// no backround in vertical panel mode
 			setBackgroundHints(NoBackground);
 		}
-		
+
 		updateSize();
 	}
 	if (constraints.testFlag(Plasma::SizeConstraint))
@@ -662,12 +667,12 @@ void
 YaWP::updateSize()
 {
 	QSize sizeHint = m_pAppletPainter->getSize(size());
-	
+
 	if (formFactor() == Plasma::Horizontal)
 	{
 		setMinimumHeight(0);
 		setMaximumHeight(QWIDGETSIZE_MAX);
-	
+
 		setMinimumWidth(sizeHint.width());
 		setMaximumWidth(sizeHint.width());
 	}
@@ -685,7 +690,7 @@ YaWP::updateSize()
 		setMaximumWidth(QWIDGETSIZE_MAX);
 		setMinimumHeight(30);
 		setMaximumHeight(QWIDGETSIZE_MAX);
-		
+
 		resize(sizeHint);
 	}
 }
@@ -773,11 +778,13 @@ YaWP::configAccepted()
 		// handle this events. Therefore there is no way for the user to hide the popup manual,
 		// unless user is adding at least one city to the applet.
 		hidePopup();
-		
+
 		// remove current tooltip
 		Plasma::ToolTipManager::self()->clearContent(this);
 	}
 	
+	m_pConfigDlg->resetChanges();
+
 	// to make sure we will change the theme
 	constraintsEvent( Plasma::FormFactorConstraint | Plasma::SizeConstraint );
 	initAppletPainter();
@@ -797,19 +804,19 @@ YaWP::mousePressEvent(QGraphicsSceneMouseEvent * event)
 	{
 		return;
 	}
-	
+
 	// every time user interacts with this plasmoid, we reset the timer
 	// to automatically switch to next location
 	stopTraverseLocationTimeout();
 
 	event->setAccepted(false);
-	
+
 	if (m_pAppletPainter->formFactor() == Plasma::Planar ||
 	    m_configData.bUseInteractivePanelWeatherIcons)
 	{
 		m_pAppletPainter->mousePressEvent(event);
 	}
-	
+
 	if (!event->isAccepted() && m_pAppletPainter->formFactor() != Plasma::Planar)
 		togglePopup();
 
@@ -829,7 +836,7 @@ YaWP::timerEvent(QTimerEvent * event)
 	else if (event->timerId() == m_iIdTraverseLocations)
 	{
 		stopTraverseLocationTimeout();
-		
+
 		int iSelectedCityIndex = (m_configData.iCityIndex + 1) % m_pWeatherModel->rowCount();
 		m_pAppletPainter->initCityChangeAnimation(iSelectedCityIndex);
 	}
@@ -842,13 +849,13 @@ YaWP::slotCityUpdate(WeatherServiceModel::ServiceUpdate updateType)
 	dStartFunct();
 	stopPendingEngineConnection();
 	const CityWeather * const pCity = m_stateMachine.currentCity();
-	
+
 	if(!pCity)
 	{
 		dEndFunct();
 		return;
 	}
-	
+
 	if( updateType.testFlag(WeatherServiceModel::CityInfoUpdate) )
 	{
 		updateCitySubMenu();
@@ -860,6 +867,8 @@ YaWP::slotCityUpdate(WeatherServiceModel::ServiceUpdate updateType)
 	if (m_pAppletPainter->formFactor() != Plasma::Planar)
 		createPanelTooltip();
 	
+	m_pOpenForecastUrl->setEnabled( !pCity->creditUrl().isNull() );
+
 	m_pAppletPainter->update();
 	dEndFunct();
 }
@@ -888,6 +897,17 @@ YaWP::about() const
 	aboutDialog->show ();
 }
 
+void
+YaWP::openForecastUrl()
+{
+	const CityWeather * city = m_stateMachine.currentCity();
+	if (!city)
+		return;
+	
+	if (!city->creditUrl().isNull())
+		Utils::OpenUrl(city->creditUrl());
+}
+
 void
 YaWP::animationFinished()
 {
@@ -904,7 +924,7 @@ YaWP::setCityIndex( int cityIndex )
 	m_stateMachine.setCurrentCityIndex( cityIndex );
 	m_configData.iCityIndex = m_stateMachine.currentCityIndex();
 	const CityWeather * city = m_stateMachine.currentCity();
-	
+
 	if (city)
 	{
 		//--- select the right menuitem in the city-submenu ---
@@ -912,10 +932,12 @@ YaWP::setCityIndex( int cityIndex )
 		int iCityIndex = m_stateMachine.currentCityIndex();
 		if (list.count() > iCityIndex)
 			list.at(iCityIndex )->setChecked( true );
-		
+
 		//--- update the paneltooltip when we are in panel mode only ---
 		if (m_pAppletPainter->formFactor() != Plasma::Planar)
 			createPanelTooltip();
+		
+		m_pOpenForecastUrl->setEnabled( !city->creditUrl().isNull() );
 	}
 	m_pAppletPainter->update();
 	dEndFunct();
@@ -931,7 +953,7 @@ void
 YaWP::slotThemeChanged()
 {
 	setDefaultFontColors();
-	
+
 	if (m_pAppletPainter->formFactor() != Plasma::Planar)
 		createPanelTooltip();
 }
@@ -946,10 +968,10 @@ YaWP::slotStartCacheCleanUp()
 		QDir::NoSort,
 		QDir::Files | QDir::NoSymLinks | QDir::Writable);
 	QFileInfoList cacheFiles = cacheDir.entryInfoList();
-	
+
 	const QDateTime currentDateTime = QDateTime::currentDateTime();
 	KUrl::List selectedCacheFiles;
-	
+
 	foreach(QFileInfo info, cacheFiles)
 	{
 		// When file is older than five days, we add it to the list of files that will be removed.
@@ -963,7 +985,7 @@ YaWP::slotStartCacheCleanUp()
 			dTracing() << "Keeping file" << info.absoluteFilePath();
 		}
 	}
-	
+
 	// Start job to remove selected files. We hide the progress information,
 	// because we do not want to bother the user with this stuff.
 	if (selectedCacheFiles.count() > 0)
@@ -1022,7 +1044,7 @@ YaWP::updateCitySubMenu()
 		action->setData( QVariant(i) );
 		m_pGrpActionCities->addAction( action );
 		m_pCitySubMenu->addAction( action );
-		
+
 		if( i == m_configData.iCityIndex )
 			action->setChecked( true );
 	}
@@ -1033,7 +1055,7 @@ void
 YaWP::initAppletPainter()
 {
 	const Plasma::FormFactor form = formFactor();
-	
+
 	if (m_pAppletPainter == 0 || m_pAppletPainter->formFactor() != form)
 	{
 		if (m_pAppletPainter != 0)
@@ -1041,7 +1063,7 @@ YaWP::initAppletPainter()
 			m_pAppletPainter->disconnect();
 			this->disconnect(m_pAppletPainter);
 			this->disconnect(m_pAppletPainter->timeLine());
-			
+
 			delete m_pAppletPainter;
 		}
 
@@ -1062,12 +1084,12 @@ YaWP::initAppletPainter()
 		connect(m_pAppletPainter, SIGNAL(signalToggleWeatherIcon(int)), this, SLOT(slotToggleWeatherIcon(int)), Qt::DirectConnection);
 		connect(m_pAppletPainter->timeLine(), SIGNAL(finished()),  this, SLOT(animationFinished()));
 	}
-	
+
 	if (m_svg.isValid())
 		m_pAppletPainter->setSvg(&m_svg);
 	if (m_customSvg.isValid())
 		m_pAppletPainter->setCustomSvg(&m_customSvg);
-	
+
 	m_pAppletPainter->setupAnimationTimeLine();
 }
 
@@ -1077,16 +1099,16 @@ YaWP::createExtenderItem()
 {
 	bool itemFound = extender()->hasItem(PANEL_DESKTOP_INTERFACE_NAME);
 	Plasma::ExtenderItem * item = 0;
-	
+
 	if (itemFound)
 		item = extender()->item(PANEL_DESKTOP_INTERFACE_NAME);
-	
+
 	// When we already have an extender item, just connect the current painter with this extender item
 	if (item != 0 && item->widget() != 0)
 	{
 		PanelDesktopInterface * popupDesktopInterface = dynamic_cast<PanelDesktopInterface *>(item->widget());
 		PanelPainter * panelPainter = dynamic_cast<PanelPainter *>(m_pAppletPainter);
-		
+
 		if (panelPainter != 0 && popupDesktopInterface != 0)
 		{
 			panelPainter->setPopupPainter(popupDesktopInterface->desktopPainter());
@@ -1121,14 +1143,14 @@ YaWP::initExtenderItem(Plasma::ExtenderItem * item)
 		popupDesktopInterface->setSvg(&m_svg);
 		popupDesktopInterface->setCustomSvg(&m_customSvg);
 		popupDesktopInterface->setupAnimationTimeLine();
-		
+
 		popupDesktopInterface->setMinimumSize(QSizeF(100, 93));
 		popupDesktopInterface->setPreferredSize(273, 255);
 
 		panelPainter->setPopupPainter(popupDesktopInterface->desktopPainter());
-		
+
 		connect( m_pWeatherModel, SIGNAL(isBusy(bool)), popupDesktopInterface, SLOT(setBusy(bool)) );
-		
+
 		item->setTitle(i18n("yaWP Desktop Interface"));
 		item->setIcon("weather-clear");
 		item->setWidget(popupDesktopInterface);
@@ -1141,7 +1163,7 @@ void
 YaWP::destroyExtenderItem()
 {
 	bool itemFound = extender()->hasItem(PANEL_DESKTOP_INTERFACE_NAME);
-	
+
 	if (itemFound)
 	{
 		extender()->item(PANEL_DESKTOP_INTERFACE_NAME)->deleteLater();
@@ -1153,7 +1175,7 @@ YaWP::createPanelTooltip()
 {
 	dStartFunct();
 	Plasma::ToolTipManager::self()->clearContent(this);
-	
+
 	//--- just exit, when we have no city or no weather information ---
 	const CityWeather * city = m_stateMachine.currentCity();
 	if( !city || city->days().count() == 0 )
@@ -1164,7 +1186,7 @@ YaWP::createPanelTooltip()
 
 	Plasma::ToolTipContent toolTipData;
 	toolTipData.setMainText( city->localizedCityString() );
-	
+
 	//--- Create the tooltip pixmap for simple layout and extended tooltip ---
 	if (!m_configData.bUseExtendedTooltip ||
 	    m_configData.extendedTooltipOptions.testFlag(Yawp::PreviewPage))
@@ -1172,46 +1194,46 @@ YaWP::createPanelTooltip()
 		DesktopPainter toolTipPainter(0, &m_configData, &m_stateMachine);
 		toolTipPainter.setSvg(&m_svg);
 		toolTipPainter.setCustomSvg(&m_customSvg);
-		
+
 		QPixmap pix;
 		if (!m_configData.bUseExtendedTooltip || city->days().count() == 1)
 			pix = toolTipPainter.createSimpleToolTip(QSize(218,0));
 		else
 			pix = toolTipPainter.createExtendedToolTip(QSize(218,0));
-		
+
 		if (!pix.isNull())
 			toolTipData.setImage(pix);
 	}
-	
+
 	//--- Create the content of the extended tooltip ---
 	if (m_configData.bUseExtendedTooltip)
 	{
 		if (!m_configData.extendedTooltipOptions.testFlag(Yawp::PreviewPage))
 		{
  			KIcon appIcon(icon());
-			toolTipData.setImage( appIcon.pixmap(48,48) ); 
+			toolTipData.setImage( appIcon.pixmap(48,48) );
 		}
-		
+
 		bool showSatellitePage
 		      = m_configData.extendedTooltipOptions.testFlag(Yawp::SatellitePage) &&
 		        m_stateMachine.hasPage(Yawp::SatellitePage);
 		QString toolTipText;
-		
+
 		if (showSatellitePage)
 			toolTipText += "<table><tr><td>";
-		
+
 		//--- Create the forecast information for today and the following two days ---
 		const int maxDay = qMin(3, city->days().count());
 		for (int dayIndex = 0; dayIndex < maxDay; ++dayIndex)
 		{
 			const YawpDay * day = city->days().at(dayIndex);
-			
+
 			// QDate returns translated nominative case of day
 			// name which is grammatically wrong at least in
 			// Russian. So fetch base English form and translate
 			// into correct one with own catalog
 			const KDateTime kDate(day->date());
-			
+
 			toolTipText += "<u>"
 			            + i18n("Forecast for %1 (%2)",
 			                   i18nc("Forecast for day",
@@ -1221,7 +1243,7 @@ YaWP::createPanelTooltip()
 			toolTipText += i18n("Day: ")
 			            + Utils::LocalizedWeatherString(day->weather().description())
 			            + "<br />";
-			
+
 			if (day->hasNightValues())
 			{
 				toolTipText += i18n("Night: ")
@@ -1253,29 +1275,29 @@ YaWP::createPanelTooltip()
 						+ "<br />";
 				}
 			}
-			
+
 			if (dayIndex+1 < maxDay)
 				toolTipText += "<br />";
 			else
 				// removes the last <br />; because this is the end of text
 				toolTipText.remove(toolTipText.length()-6, 6);
 		}
-		
+
 		if (showSatellitePage)
 		{
 			toolTipData.addResource(
 				Plasma::ToolTipContent::ImageResource,
 				QUrl("wicon://satelite"),
 				QVariant(city->satelliteImage().scaledToWidth(218, Qt::SmoothTransformation)));
-			
+
 			toolTipText += "</td><td valign=\"top\">";
 			toolTipText += "<img src=\"wicon://satelite\"/>";
-			toolTipText += "</td></tr></table>";		  
+			toolTipText += "</td></tr></table>";
 		}
-		
+
 		toolTipData.setSubText(toolTipText);
 	}
-	
+
 	toolTipData.setAutohide(false);
 	Plasma::ToolTipManager::self()->setContent(this, toolTipData);
 
