@@ -226,8 +226,8 @@ public:
 
 	const QString & credit() const { return m_sCredit; }
 	void setCredit(const QString & sCredit) { m_sCredit = sCredit; }
-	const QString & creditURL() const { return m_sCreditURL; }
-	void setCreditURL(const QString & sCreditURL) { m_sCreditURL = sCreditURL; }
+	const QString & creditUrl() const { return m_sCreditUrl; }
+	void setCreditUrl(const QString & sCreditUrl) { m_sCreditUrl = sCreditUrl; }
 	
 	/*** Returns the current local time depending on specifed timezone. */
 	QDateTime localTime() const;
@@ -264,7 +264,7 @@ private:
 	QDateTime         m_observationPeriode;
 
 	QString           m_sCredit;
-	QString           m_sCreditURL;
+	QString           m_sCreditUrl;
 
 	QImage            m_satelliteImage;
 };
