@@ -40,7 +40,6 @@
 //--- LOCAL ---
 #include "../config.h"
 #include "ion_accuweather.h"
-#include "accuweather_configloader.h"
 #include "logger/streamlogger.h"
 
 //--- QT4 ---
@@ -65,10 +64,6 @@ static const char vWeekdays [7][4] = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat",
 
 struct AccuWeatherIon::Private
 {
-	QMutex                            syncMutex;
-	AccuWeatherConfigMap    *         configMap;
-	AccuWeatherConfigLoader *         currentConfigLoader;
-  
 	QMap<QString, ConditionIcons>     vConditionList;
 
 	QHash<QString, KJob *>            vActiveJobs;
@@ -93,12 +88,8 @@ AccuWeatherIon::AccuWeatherIon( QObject * parent, const QVariantList & args )
 	  d( new Private )
 {
 	Q_UNUSED(args)
-	d->currentConfigLoader = 0;
-	d->configMap = 0;
-	
 	dInfo() << "AccuWeatherIon" << YAWP_VERSION_STRING << "compiled at" << __DATE__ << __TIME__ << "for KDE" << KDE_VERSION_STRING;
 
-
 #if defined(MIN_POLL_INTERVAL)
 	setMinimumPollingInterval(MIN_POLL_INTERVAL);
 #endif 
@@ -153,22 +144,6 @@ AccuWeatherIon::~AccuWeatherIon()
 	dStartFunct();
 	
 	cleanup();
-	
-	// We can not use QMutexLocker here, because we will delete d, which contains d->syncMutex!!!
-	//
-	d->syncMutex.lock();
-	if( d->currentConfigLoader != 0 )
-	{
-		if( d->currentConfigLoader->isRunning() )
-			d->currentConfigLoader->wait();
-		
-		delete d->currentConfigLoader;
-		d->currentConfigLoader = 0;
-	}
-	if( d->configMap != 0 )
-		delete d->configMap;
-	d->syncMutex.unlock();
-	
 	delete d;
 	
 	dEndFunct();
@@ -177,45 +152,7 @@ AccuWeatherIon::~AccuWeatherIon()
 void
 AccuWeatherIon::init()
 {
-	dStartFunct();
-	
-	bool bInitialized = true;
-	
-	d->syncMutex.lock();
-	if( d->currentConfigLoader == 0 && d->configMap == 0 )
-	{
-		bInitialized = false;
-		
-		d->currentConfigLoader = new AccuWeatherConfigLoader();
-		connect(d->currentConfigLoader, SIGNAL(finished()), this, SLOT(slotConfigLoaderFinished()));
-	
-		d->currentConfigLoader->start(QThread::LowPriority);
-	}
-	d->syncMutex.unlock();
-	
-	if (bInitialized)
-		setInitialized(true);
-	
-	dEndFunct();
-}
-
-void
-AccuWeatherIon::slotConfigLoaderFinished()
-{
-	dStartFunct();
-	
-	d->syncMutex.lock();
-	if( d->currentConfigLoader != 0 )
-	{
-		d->configMap = d->currentConfigLoader->configMap();
-		delete d->currentConfigLoader;
-		d->currentConfigLoader = 0;
-	}
-	d->syncMutex.unlock();
-	
 	setInitialized(true);
-	
-	dEndFunct();
 }
 
 bool
@@ -456,13 +393,10 @@ AccuWeatherIon::readSearchXmlData( const QString & location, const QString & sou
 void
 AccuWeatherIon::parseSearchLocations( const QString & location, const QString & source, QXmlStreamReader & xml )
 {
-	Q_ASSERT_X( xml.isStartElement() && xml.name() == "citylist",
-				"AccuWeather Ion",
-	            "AccuWeatherIon::parseSearchLocations - current tag is wrong!" );
+	dStartFunct();
 
 	uint iCounter = 0;
 	QString sLocations;
-	dStartFunct();
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
@@ -593,27 +527,21 @@ AccuWeatherIon::slotJobFinished( KJob * pJob )
 
 		readWeatherXmlData( pXmlData->xmlReader, *pWeather );
 
-		if( pImageData )
+		if (pImageData && !pImageData->bReady)
 		{
-			if( pImageData->bReady )
-			{
-				//--- Update the source and disconnect from the image data ---
-				updateWeatherSource( *pWeather, pImageData );
-				pImageData->iLocationCounter -= 1;
-			}
-			else
-			{
-				//  The attached image data has not been downloaded yet, so we store the weather data
- 				//  to the pending update sources.
-				pImageData->vWeatherSources.append( pWeather );
-			}
+			//  The attached image data has not been downloaded yet, so we store the weather data
+			//  to the pending update sources.
+			pImageData->vWeatherSources.append( pWeather );
 		}
 		else
 		{
-			//--- There is no image data attached to the location, so we just update the source. ---
-			updateWeatherSource( *pWeather, NULL );
+			//--- Update the source and disconnect from the image data ---
+			updateWeatherSource( *pWeather, pImageData );
 			qDeleteAll( pWeather->vForecasts.begin(), pWeather->vForecasts.end() );
 			delete pWeather;
+			
+			if (pImageData)
+				pImageData->iLocationCounter -= 1;
 		}
 	}
 	//--- when we do not need the image anymore and the image job has been completed remove the image data ---
@@ -687,10 +615,6 @@ AccuWeatherIon::readWeatherXmlData( QXmlStreamReader & xml, WeatherData & weathe
 void
 AccuWeatherIon::readUnits( QXmlStreamReader & xml, WeatherData & weather )
 {
-	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("units") == 0,
-	            "AccuWeather Ion",
-	            "AccuWeatherIon::readUnits - wrong tag to parse!" );
-
 	dStartFunct();
 	while( !xml.atEnd() )
 	{
@@ -733,9 +657,6 @@ AccuWeatherIon::readUnits( QXmlStreamReader & xml, WeatherData & weather )
 void
 AccuWeatherIon::readLocal( QXmlStreamReader & xml, WeatherData & weather )
 {
-	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("local") == 0,
-	            "AccuWeather Ion",
-	            "AccuWeatherIon::readLocal - wrong tag to parse!" );
 	dStartFunct();
 	while( !xml.atEnd() )
 	{
@@ -748,9 +669,9 @@ AccuWeatherIon::readLocal( QXmlStreamReader & xml, WeatherData & weather )
 				weather.sLatitude = xml.readElementText();
 			else if( xml.name().compare("lon") == 0 )
 				weather.sLongitude = xml.readElementText();
-			else if( xml.name().compare("time") == 0 )
+/*			else if( xml.name().compare("time") == 0 )
 				weather.sObservationPeriode = xml.readElementText();
-			else if( xml.name().compare("timeZone") == 0 )
+*/			else if( xml.name().compare("timeZone") == 0 )
 			{
 				QString sTimeZone(xml.readElementText());
 				int iPos( sTimeZone.indexOf(QChar(':')) );
@@ -771,10 +692,6 @@ AccuWeatherIon::readLocal( QXmlStreamReader & xml, WeatherData & weather )
 void
 AccuWeatherIon::readCurrentConditions( QXmlStreamReader & xml, WeatherData & weather )
 {
-	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("currentconditions") == 0,
-	            "AccuWeather Ion",
-	            "AccuWeatherIon::readCurrentConditions - wrong tag to parse!" );
-	
 	dStartFunct();
 	while( !xml.atEnd() )
 	{
@@ -784,7 +701,9 @@ AccuWeatherIon::readCurrentConditions( QXmlStreamReader & xml, WeatherData & wea
 		else if( xml.isStartElement() )
 		{
 //			dDebug() << xml.name().toString();
-			if( xml.name().compare("temperature") == 0 )
+			if( xml.name().compare("url") == 0)
+				weather.forecastUrl = xml.readElementText();
+			else if( xml.name().compare("temperature") == 0 )
 				weather.temperature = xml.readElementText();
 			else if( xml.name().compare("realfeel") == 0 )
 				weather.realfeel = xml.readElementText();
@@ -822,6 +741,13 @@ AccuWeatherIon::readCurrentConditions( QXmlStreamReader & xml, WeatherData & wea
 				weather.uvIndex = xml.attributes().value("index").toString();
 				weather.uvRating = xml.readElementText();
 			}
+			
+			// this tag contains the same value, than local/time but should make content more clear
+			else if (xml.name().compare("observationtime") == 0)
+			{
+				QString timeString = xml.readElementText();
+				weather.observationPeriode = QTime::fromString(timeString, "h:m ap");
+			}
 		}
 	}
 	if( xml.hasError() )
@@ -832,9 +758,6 @@ AccuWeatherIon::readCurrentConditions( QXmlStreamReader & xml, WeatherData & wea
 void
 AccuWeatherIon::readForecastConditions( QXmlStreamReader & xml, ForecastDay & forecast )
 {
-	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("day") == 0,
-	            "AccuWeather Ion",
-	            "AccuWeatherIon::readForecastConditions - current tag is wrong!" );
 	dStartFunct();
 	while( !xml.atEnd() )
 	{
@@ -873,11 +796,6 @@ AccuWeatherIon::readForecastConditions( QXmlStreamReader & xml, ForecastDay & fo
 void
 AccuWeatherIon::readWeatherConditions( QXmlStreamReader & xml, ForecastConditions & conditions )
 {
-	Q_ASSERT_X( xml.isStartElement() &&
-			(xml.name().compare("daytime") == 0 || xml.name().compare("nighttime") == 0),
-		"AccuWeather Ion",
-		"AccuWeatherIon::readForecastConditions - current tag is wrong!" );
-
 	dStartFunct();
 	while( !xml.atEnd() )
 	{
@@ -968,26 +886,30 @@ AccuWeatherIon::updateWeatherSource( const WeatherData & weather, const ImageDat
 
 		QTime sunrise( QTime::fromString(weather.vForecasts.at(0)->sunrise, "h:mm AP") );
 		QTime sunset( QTime::fromString(weather.vForecasts.at(0)->sunset, "h:mm AP") );
-		if( sunrise.isValid() && sunset.isValid() )
+		if( sunrise.isValid() && sunset.isValid() && currDate.isValid())
 		{
 #if KDE_VERSION_MINOR <= 2
 			setData( weather.source, "Sunrise At", sunrise.toString("h:m") );
 			setData( weather.source, "Sunset At", sunset.toString("h:m") );
 #else
-			KDateTime dateTime( currDate, sunrise, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
-			if( dateTime.isValid() )
-				setData( weather.source, "Sunrise At", dateTime.toString("%A %B %e, %Y at %H:%M %Z") );
-		
-			dateTime = KDateTime( currDate, sunset, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
-			if( dateTime.isValid() )
-				setData( weather.source, "Sunset At", dateTime.toString("%A %B %e, %Y at %H:%M %Z") );
+			// Format of sunrise and sunset: "Tuesday November 29, 2011 at 07:43"
+			QDateTime dateTime(currDate, sunrise);
+			setData(weather.source, "Sunrise At", QString("%1, %2 at %3")
+				.arg(dateTime.toString("dddd MMMM dd"))
+				.arg(dateTime.date().year())
+				.arg(dateTime.toString("HH:mm")));
+			
+			dateTime.setTime(sunset);
+			setData(weather.source, "Sunset At", QString("%1, %2 at %3")
+				.arg(dateTime.toString("dddd MMMM dd"))
+				.arg(dateTime.date().year())
+				.arg(dateTime.toString("HH:mm")));
 #endif
 		}
 
-		QTime observationPeriode( QTime::fromString(weather.sObservationPeriode, "h:mm") );
-		if( observationPeriode.isValid() )
+		if( weather.observationPeriode.isValid() )
 		{
-			KDateTime dateTime( currDate, observationPeriode, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
+			KDateTime dateTime( currDate, weather.observationPeriode, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
 			if( dateTime.isValid() )
 				setData( weather.source, "Observation Period", dateTime.toString("%d.%m.%Y @ %H:%M") );
 		}
@@ -1016,12 +938,10 @@ AccuWeatherIon::updateWeatherSource( const WeatherData & weather, const ImageDat
 		setData( weather.source, "Satellite Map", pImageData->image );
 
 	setData( weather.source, "Credit", i18n("Supported by AccuWeather") );
-	setData( weather.source, "Credit Url", "http://www.accuweather.com" );
 	
-/*	Plasma::DataContainer * pCon = containerForSource(weather.source);
-	if( pCon )
-		dTracing() << pCon->data();
-*/
+	if (!weather.forecastUrl.isEmpty())
+		setData( weather.source, "Credit Url", weather.forecastUrl );
+	
 	dEndFunct();
 }
 
@@ -1084,109 +1004,82 @@ AccuWeatherIon::updateSun( const QString & source,
 QByteArray
 AccuWeatherIon::getImageUrl( const QString & sLocationCode ) const
 {
-	QByteArray sUrl = "";
-	
 	QStringList lst = sLocationCode.split('|',QString::SkipEmptyParts);
-	QString firstKey;
-	QString secondKey;
-	QString thirdKey;
+
+	//--- When lst contains just one entry, we assume it is a city in the United States ---
+	if (lst.count() == 1)
+		return QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/EI/iseun.jpg");
+
+	QByteArray sUrl;
 	
-	if( lst.count() > 2 )
+	if (lst[0].compare("EUR") == 0)
+		sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iseurm.jpg");
+	
+	else if (lst[0].compare("SAM") == 0)
 	{
-		if( lst.count() >= 3 )
-		{
-			QString locationCode = lst[2].left(4);
-			firstKey = AccuWeatherConfigLoader::createLocationCode(lst[0], lst[1], locationCode);
-		}
-		secondKey = AccuWeatherConfigLoader::createLocationCode(lst[0], lst[1]);
-		thirdKey = AccuWeatherConfigLoader::createLocationCode(lst[0]);
-	}
-	else if( sLocationCode.length() > 2 )	// we assume it is a US country
-	{
-		QString areaCode = "US";
-		QString countryCode = sLocationCode.left(2);
-		QString locationCode = "";
-		
-		if(sLocationCode.length() >= 4)
-			locationCode = sLocationCode.mid(2,2);
-		
-		firstKey = AccuWeatherConfigLoader::createLocationCode(areaCode, countryCode, locationCode);
-		secondKey = AccuWeatherConfigLoader::createLocationCode(areaCode, countryCode);
-		thirdKey = AccuWeatherConfigLoader::createLocationCode(areaCode);
+		if (lst[1].compare("AR") == 0 || lst[1].compare("CL") == 0)
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iscsam.jpg");
+		else
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isnsam.jpg");
 	}
 	
-	d->syncMutex.lock();
-	if( d->configMap != 0 )	// config map has been loaded and it is save to exit the data
+	else if (lst[0].compare("NAM") == 0)
 	{
-		if( !firstKey.isEmpty() && d->configMap->contains(firstKey) )
-			sUrl = d->configMap->value(firstKey);
-		else if( !secondKey.isEmpty() && d->configMap->contains(secondKey) )
-			sUrl = d->configMap->value(secondKey);
-		else if( !thirdKey.isEmpty() && d->configMap->contains(thirdKey) )
-			sUrl = d->configMap->value(thirdKey);
+		if (lst[1].compare("CA") == 0)
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/ir/iscanm.jpg");
+		else if (lst[1].compare("MX") == 0)
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/ismex.jpg");
+		else
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/ir/isun.jpg");
 	}
-	d->syncMutex.unlock();
 
-	dDebug() << "Url for" << sLocationCode << sUrl;
-	return sUrl;
-
-/*
-	//--- city of the united states ---
-	if( sLocationCode.at(3) != '|' )
-		sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/EI/iseun.jpg");
-	else
+	else if (lst[0].compare("CAC") == 0)
+		sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iscar.jpg");
+	
+	else if (lst[0].compare("OCN") == 0)
+		sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isaust.jpg");
+	
+	else if (lst[0].compare("ASI") == 0)
 	{
-		QString area = sLocationCode.mid(0,3);
-		QString country = sLocationCode.mid(4,2);
-
-		if( area == QLatin1String("EUR") )
-			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iseurm.jpg");
-		else if( area == QLatin1String("SAM") )
+		if (lst[1].compare("IN") == 0)
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isindia.jpg");
+		else if (lst[1].compare("RU") == 0)
 		{
-			if( country == QLatin1String("AR") || country == QLatin1String("CL") )
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iscsam.jpg");
-			else
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isnsam.jpg");
-		}
-		else if(area == QLatin1String("NAM") )
-		{
-			if( country == QLatin1String("CA") )
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/ir/iscanm.jpg");
-			else if( country == QLatin1String("MX") )
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/ismex.jpg");
-			else
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/ir/isun.jpg");
-		}
-		else if( area == QLatin1String("CAC") )
-			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iscar.jpg");
-		else if( area == QLatin1String("OCN") )
-			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isaust.jpg");
-		else if( area == QLatin1String("ASI") )
-		{
-			if (country == QLatin1String("IN") )
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isindia.jpg");
-			else if( country == QLatin1String("RU") )
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iseurm.jpg");
-			else
+			QString code = lst.count() >= 3 ? lst[2].left(4) : "";
+			
+			if (code.compare("RS00") == 0 ||
+			    code.compare("RS01") == 0 ||
+			    code.compare("RS02") == 0 ||
+			    code.compare("RS03") == 0)
+			{
 				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isasia.jpg");
-		}
-		else if( area == QLatin1String("MEA") )
-			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/ismide.jpg");
-		else if( area == QLatin1String("AFR") )
-		{
-			if( country == QLatin1String("MA") ||
-			    country == QLatin1String("DZ") ||
-			    country == QLatin1String("TN") ||
-			    country == QLatin1String("LY") ||
-			    country == QLatin1String("EG") )
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isafrn.jpg");
+			}
 			else
-				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isafrs.jpg");
+				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iseurm.jpg");
 		}
+		else
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isasia.jpg");
 	}
-*/
-}
 
+	else if (lst[0].compare("MEA") == 0)
+		sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/ismide.jpg");
+	
+	else if (lst[0].compare("AFR") == 0)
+	{
+		if (lst[1].compare("MA") == 0 ||
+		    lst[1].compare("DZ") == 0 ||
+		    lst[1].compare("TN") == 0 ||
+		    lst[1].compare("LY") == 0 ||
+		    lst[1].compare("EG") == 0)
+		{
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isafrn.jpg");
+		}
+		else
+			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isafrs.jpg");
+	}
+	
+	return sUrl;
+}
 
 void
 AccuWeatherIon::connectWithImageData( const QByteArray & imageUrl )
@@ -1224,9 +1117,7 @@ AccuWeatherIon::image_slotDataArrived( KIO::Job * pJob, const QByteArray & data
 {
 	if( data.isEmpty() || !d->vImageJobs.contains(pJob) )
 		return;
-//	dStartFunct();
 	d->vImageJobs[pJob]->fetchedData.append( data );
-//	dEndFunct();
 }
 
 void
