@@ -88,12 +88,14 @@ struct XmlServiceData
 	QString            sLocation;
 	QString            sSource;
 	QString            sServiceId;
+	
+	KIO::TransferJob * pJob;
 };
 
 struct XmlForecastDay
 {
-	QString sDayName;	// equals one of the following strings: Today, Tonight, Mon, Tue, Wed, Thu, Fri, Sat, Sun
-	QString sDescription;	// weather-description
+	QString sDayName;           // equals one of the following strings: Today, Tonight, Mon, Tue, Wed, Thu, Fri, Sat, Sun
+	QString sDescription;       // weather-description
 	QString sIcon;
 	QString sHighTemp;
 	QString sLowTemp;
@@ -101,8 +103,13 @@ struct XmlForecastDay
 
 struct XmlWeatherData
 {
-	KLocale::MeasureSystem	measureSystem;
-	QString sObservationTime;				// date and time when the service did the last updated on the weather-data
+	KLocale::MeasureSystem measureSystem;
+	QString sForecastUrl;
+	QString sObservationTime;   // date and time when the service did the last updated on the weather-data
+	QUrl imageUrl;
+	
+	QString sSource;
+	QString sLocation;
 	
 	QString sCurrentIcon;
 	QString sDayCondition;
@@ -130,19 +137,50 @@ struct XmlWeatherData
 	 *  We will return the results to the widget, when all jobs have been completed.
 	 */
 	short iActiveJobs;
-	short iSuccededJobs;
 };
 
+struct ImageData
+{
+	QByteArray fetchedData;
+	QUrl imageUrl;
+	QImage image;
+
+	/*  Every time, a weather job is ready, we check if the image job is ready as well in order to
+	 *  order to update the source:
+	 *  ImageJob is ready:	update source immedatly and delete the weatherData in the list
+	 *                        since we do not need them anymore
+	 *  otherwise:			we add the weatherData to the list vWeatherSources.
+	 *                         When imagejob is ready, than we will update all locations in this list,
+	 *                         delete the job and keep the source until all weather has been updated.
+	 */
+	bool bReady;
+
+	/*  counter of all cities that relay on this image are not ready yet */
+	int locationCounter;
+
+	/*  List contains the data of location where the job has been already completed
+	 *  and the data are just waiting for the ImageJob to update.
+	 */
+	QList<XmlWeatherData *> vWeatherSources;
+};
+
+
 
 struct WundergroundIon::Private
 {
 	QMap<QString, ConditionIcons>     vIconConditionMap;
 	QHash<QString, QString>           vCountryStateMap;
 	
-	QHash<KJob *,  XmlServiceData *>  vServiceJobs;	// contains all running jobs
-	QHash<QString, XmlLookupResult *> vLookupResults;	// key is the location we're looking for
-	QHash<QString, XmlWeatherData *>  vWeatherData;	// key is ServiceId (e.g.: airport:EDDC)
+	QHash<QString, XmlServiceData *>  vServiceJobs;       // contains all running jobs
+	QHash<QString, XmlLookupResult *> vLookupResults;     // key is the location we're looking for
+	QHash<QString, XmlWeatherData *>  vWeatherData;       // key is ServiceId (e.g.: airport:EDDC)
 	
+	//--- Image jobs ---
+	// We store the same ImageData in both Hashmaps, therefore we have to make sure,
+	// we delete this data just once.
+	QHash<QUrl, ImageData *>          vImageData;         // Satellite url is the key to the ImageData
+	QHash<KJob *, ImageData *>        vImageJobs;
+
 #if KDE_IS_VERSION(4,4,80)
 	QStringList                       sourcesToReset;
 #endif
@@ -202,6 +240,7 @@ WundergroundIon::WundergroundIon( QObject * parent, const QVariantList & args )
 	 *  Only this names needs to be inserted in this map (in lower letters) to map this names in the ion.
 	 */
 	d->vCountryStateMap["dl"] = "Germany";
+	d->vCountryStateMap["ci"] = "China";
 }
 
 WundergroundIon::~WundergroundIon()
@@ -243,10 +282,10 @@ WundergroundIon::cleanup()
 	dStartFunct();
 	
 	dDebug() << "Delete Jobs";
-	QHash<KJob *, XmlServiceData *>::iterator itService;
+	QHash<QString, XmlServiceData *>::iterator itService;
 	for( itService = d->vServiceJobs.begin(); itService != d->vServiceJobs.end(); ++itService )
 	{
-		itService.key()->kill( KJob::Quietly );
+		itService.value()->pJob->kill( KJob::Quietly );
 		delete itService.value();
 	}
 	d->vServiceJobs.clear();
@@ -269,6 +308,24 @@ WundergroundIon::cleanup()
 	}
 	d->vWeatherData.clear();
 	
+	dDebug() << "Delete image data";
+	QHash<QUrl, ImageData *>::iterator itImageData;
+	for (itImageData = d->vImageData.begin(); itImageData != d->vImageData.end(); ++itImageData)
+	{
+		delete itImageData.value();
+	}
+	d->vImageData.clear();
+	
+	dDebug() << "Delete Image Jobs";
+	QHash<KJob *, ImageData *>::iterator itImageJob;
+	for (itImageJob = d->vImageJobs.begin(); itImageJob != d->vImageJobs.end(); ++itImageJob)
+	{
+		// !!! Do not delete the value of the QHashMap, because this is the same as in vImageData
+		
+		itImageJob.key()->kill( KJob::Quietly );
+	}
+	d->vImageJobs.clear();
+	
 	dEndFunct();
 }
 
@@ -324,21 +381,36 @@ void
 WundergroundIon::setup_findPlace( const QString & sLocation, const QString & sSource, const QString & sPath )
 {
 	dStartFunct();
+	
+	//--- Check if we already have a job for this location that is downloading the specific path ---
+	/*   Search request "China" contains the following two entries:
+	 *      <location type="CITY"><name>China, Mexico</name><link>/US/TX/China.html</link></location>
+	 *      <location type="CITY"><name>China, Texas</name><link>/US/TX/China.html</link></location>
+	 *   in this case we would download that page twice!! Therefore we save the jobs by name.
+	 *   The job-name will be created from location and path.
+	 */
+	QString sKey = QString("%1|%2|%3").arg(ActionValidate).arg(sLocation).arg(sPath);
+	if (d->vServiceJobs.contains(sKey))
+	{
+		dEndFunct();
+		return;
+	}
 	QUrl url(GeoLookupXML+(sPath.isEmpty() ? "/index.xml" : sPath),  QUrl::StrictMode);
 	if( sPath.isEmpty() )
 		url.addEncodedQueryItem("query", QUrl::toPercentEncoding(sLocation) );
-	dDebug() << url;
 	
 	KIO::TransferJob * pJob = KIO::get( url, KIO::Reload, KIO::HideProgressInfo );
 	if( pJob )
 	{
 		pJob->addMetaData("cookies", "none"); // Disable displaying cookies
+		pJob->setObjectName(sKey);
 
 		struct XmlServiceData * pXmlData = new XmlServiceData;
 		pXmlData->sLocation = sLocation;
 		pXmlData->sSource = sSource;
+		pXmlData->pJob = pJob;
 
-		d->vServiceJobs.insert( pJob, pXmlData );
+		d->vServiceJobs.insert( sKey, pXmlData );
 		if( !d->vLookupResults.contains(sLocation) )
 		{
 			struct XmlLookupResult * pResult = new XmlLookupResult;
@@ -364,19 +436,23 @@ WundergroundIon::setup_findPlace( const QString & sLocation, const QString & sSo
 void
 WundergroundIon::setup_slotDataArrived( KIO::Job * job, const QByteArray & data )
 {
-	if( data.isEmpty() || !d->vServiceJobs.contains(job) )
+	if( data.isEmpty() || !d->vServiceJobs.contains(job->objectName()) )
 		return;
 	QString sData(data);
-	d->vServiceJobs[job]->xmlReader.addData( sData.toLatin1() );
+	d->vServiceJobs[job->objectName()]->xmlReader.addData( sData.toLatin1() );
 }
 
 void
 WundergroundIon::setup_slotJobFinished( KJob * job )
 {
-	if( !d->vServiceJobs.contains(job) )
-		return;
 	dStartFunct();
-	struct XmlServiceData * pXmlData = d->vServiceJobs[job];
+	if( !d->vServiceJobs.contains(job->objectName()) )
+	{
+		dEndFunct();
+		return;
+	}
+	
+	struct XmlServiceData * pXmlData = d->vServiceJobs[job->objectName()];
 
 	if( d->vLookupResults.contains(pXmlData->sLocation) )
 	{
@@ -430,7 +506,7 @@ WundergroundIon::setup_slotJobFinished( KJob * job )
 //		else
 //			dDebug() << "Active Childjobs:" << pSearchResult->iActiveChildJobs;
 	}
-	d->vServiceJobs.remove( job );
+	d->vServiceJobs.remove( job->objectName() );
 	job->deleteLater();
 	delete pXmlData;
 
@@ -526,53 +602,142 @@ WundergroundIon::setup_readLookupData( const QString & sLocation,
 void
 WundergroundIon::slotDataArrived( KIO::Job * job, const QByteArray & data )
 {
-	if( data.isEmpty() || !d->vServiceJobs.contains(job) )
+	if( data.isEmpty() || !d->vServiceJobs.contains(job->objectName()) )
 		return;
 	QString sData(data);
-	d->vServiceJobs[job]->xmlReader.addData( sData.toLatin1() );
+	d->vServiceJobs[job->objectName()]->xmlReader.addData( sData.toLatin1() );
 }
 
 void
 WundergroundIon::slotJobFinished( KJob * job )
 {
-	if( !d->vServiceJobs.contains(job) )
-		return;
 	dStartFunct();
-	struct XmlServiceData * pXmlData = d->vServiceJobs[job];
+	if( !d->vServiceJobs.contains(job->objectName()) )
+	{
+		dEndFunct();
+		return;
+	}
+	
+	struct XmlServiceData * pXmlData = d->vServiceJobs[job->objectName()];
+	
 	if( d->vWeatherData.contains(pXmlData->sServiceId) )
 	{
 		XmlWeatherData * pWeatherData = d->vWeatherData[pXmlData->sServiceId];
-	
-		if( job->error() != 0 )
+
+		if (job->error() != 0)
 		{
 			dWarning()  << pXmlData->sSource << job->errorString();
 		}
 		else 
 		{
-			if( job->objectName().compare(XmlDataCurrentObservation) == 0 )
+			if (job->objectName().startsWith(XmlDataCurrentObservation))
 				readCurrentObservation(pXmlData->xmlReader, *pWeatherData);
-			else if( job->objectName().compare(XmlDataForecast) == 0 )
+			else if (job->objectName().startsWith(XmlDataForecast))
 				readWeatherForecast(pXmlData->xmlReader, *pWeatherData);
-			
-			pWeatherData->iSuccededJobs += 1;
 		}	
 		pWeatherData->iActiveJobs -= 1;
 		dDebug() << "active jobs for " << pXmlData->sServiceId << pWeatherData->iActiveJobs;
 			
-		if( pWeatherData->iActiveJobs <= 0 && pWeatherData->iSuccededJobs > 0 )
+		if (pWeatherData->iActiveJobs <= 0)
 		{
-			updateWeatherSource( *pWeatherData, pXmlData->sSource, pXmlData->sLocation );
-			
 			d->vWeatherData.remove(pXmlData->sServiceId);
-			qDeleteAll(pWeatherData->vForecasts.begin(), pWeatherData->vForecasts.end());
-			delete pWeatherData;
+
+			ImageData  * pImageData = NULL;
+			if( !pWeatherData->imageUrl.isEmpty() && d->vImageData.contains(pWeatherData->imageUrl) )
+				pImageData = d->vImageData[pWeatherData->imageUrl];
+			
+			if (pImageData && !pImageData->bReady)
+			{
+				//  The attached image data has not been downloaded yet, so we store the weather data
+				//  to the pending update sources.
+				pImageData->vWeatherSources.append( pWeatherData );
+			}
+			else
+			{
+				updateWeatherSource( *pWeatherData, pImageData );
+				
+				d->vWeatherData.remove(pXmlData->sServiceId);
+				qDeleteAll(pWeatherData->vForecasts.begin(), pWeatherData->vForecasts.end());
+				delete pWeatherData;
+				
+				if (pImageData)
+				{
+					pImageData->locationCounter -= 1;
+					
+					if (pImageData->locationCounter <= 0)
+					{
+						d->vImageData.remove( pImageData->imageUrl );
+						delete pImageData;
+					}
+				}
+			}
 		}
 	}
 	
-	d->vServiceJobs.remove(job);
+	d->vServiceJobs.remove(job->objectName());
 	delete pXmlData;
 	job->deleteLater();
 
+	dDebug() << "Service Jobs:   " << d->vServiceJobs.count();
+	dDebug() << "Weather Data:   " << d->vWeatherData.count();
+	dDebug() << "Image Jobs:     " << d->vImageJobs.count();
+	dDebug() << "Image Data:     " << d->vImageData.count();
+
+	dEndFunct();
+}
+
+void
+WundergroundIon::slotImageDataArrived( KIO::Job * pJob, const QByteArray & data )
+{
+	if( data.isEmpty() || !d->vImageJobs.contains(pJob) )
+		return;
+	d->vImageJobs[pJob]->fetchedData.append( data );
+}
+
+void
+WundergroundIon::slotImageJobFinished( KJob * pJob )
+{
+	if( !d->vImageJobs.contains(pJob) )
+		return;
+	
+	dStartFunct();
+	struct ImageData * pImageData = d->vImageJobs[pJob];
+	pImageData->bReady = true;
+
+	if( pJob->error() == 0 )
+	{
+		pImageData->image.loadFromData( pImageData->fetchedData, "PNG" );
+	}
+	else
+	{
+		dWarning() << pJob->errorString();
+	}
+	pImageData->fetchedData.clear();
+
+	//--- update all pending weather sources ---
+	while( pImageData->vWeatherSources.count() > 0 )
+	{
+		XmlWeatherData * pWeather = pImageData->vWeatherSources.takeFirst();
+		updateWeatherSource( *pWeather, pImageData );
+		qDeleteAll( pWeather->vForecasts.begin(), pWeather->vForecasts.end() );
+		delete pWeather;
+		pImageData->locationCounter -= 1;
+	}
+
+	d->vImageJobs.remove( pJob );
+	pJob->deleteLater();
+	
+	if( pImageData->locationCounter <= 0 )
+	{
+		d->vImageData.remove( pImageData->imageUrl );
+		delete pImageData;
+	}
+
+	dDebug() << "Service Jobs:   " << d->vServiceJobs.count();
+	dDebug() << "Weather Data:   " << d->vWeatherData.count();
+	dDebug() << "Image Jobs:     " << d->vImageJobs.count();
+	dDebug() << "Image Data:     " << d->vImageData.count();
+
 	dEndFunct();
 }
 
@@ -583,14 +748,21 @@ WundergroundIon::getWeatherData( const QString & sLocation, const QString & sSer
 
 	QStringList vTokens( sServiceId.split(':') );
 	if( vTokens.count() != 2 )
+	{
+		dEndFunct();
 		return false;
+	}
 	
 	int iJobCounter(0);
 	QUrl urlObservation;
+	bool bAirportStation = false;
+	
 	if( vTokens.at(0).compare("airport") == 0 )
 	{
 		urlObservation.setUrl("http://api.wunderground.com/auto/wui/geo/WXCurrentObXML/index.xml", QUrl::StrictMode);
 		urlObservation.addEncodedQueryItem("query", QUrl::toPercentEncoding(vTokens.at(1)));
+
+		bAirportStation = true;
 	}
 	else if( vTokens.at(0).compare("pws") == 0 )
 	{
@@ -604,15 +776,16 @@ WundergroundIon::getWeatherData( const QString & sLocation, const QString & sSer
 	if( pJobObservation )
 	{
 		iJobCounter += 1;
-		pJobObservation->setObjectName(XmlDataCurrentObservation);
 		pJobObservation->addMetaData("cookies", "none"); // Disable displaying cookies
+		pJobObservation->setObjectName(QString("%1|%2").arg(XmlDataCurrentObservation).arg(sServiceId));
 		
 		struct XmlServiceData * pXmlData = new XmlServiceData;
 		pXmlData->sLocation = sLocation;
 		pXmlData->sServiceId = sServiceId;
 		pXmlData->sSource = sSource;
+		pXmlData->pJob = pJobObservation;
 		
-		d->vServiceJobs.insert( pJobObservation, pXmlData );
+		d->vServiceJobs.insert( pJobObservation->objectName(), pXmlData );
 
 		connect( pJobObservation, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
 			SLOT(slotDataArrived(KIO::Job *, const QByteArray &)) );
@@ -620,35 +793,40 @@ WundergroundIon::getWeatherData( const QString & sLocation, const QString & sSer
 	}
 	
 	/*** download forecast information ***/
-	QUrl urlForecast("http://api.wunderground.com/auto/wui/geo/ForecastXML/index.xml");
-	urlForecast.addEncodedQueryItem("query", QUrl::toPercentEncoding(vTokens.at(1)));
-	KIO::TransferJob * pJobForecast = KIO::get( urlForecast, KIO::Reload, KIO::HideProgressInfo );
-	if( pJobForecast )
+	if (bAirportStation)
 	{
-		iJobCounter += 1;
-		pJobForecast->setObjectName(XmlDataForecast);
-		pJobForecast->addMetaData("cookies", "none"); // Disable displaying cookies
-		
-		struct XmlServiceData * pXmlData = new XmlServiceData;
-		pXmlData->sLocation = sLocation;
-		pXmlData->sServiceId = sServiceId;
-		pXmlData->sSource = sSource;
-		
-		d->vServiceJobs.insert( pJobForecast, pXmlData );
+		QUrl urlForecast("http://api.wunderground.com/auto/wui/geo/ForecastXML/index.xml");
+		urlForecast.addEncodedQueryItem("query", QUrl::toPercentEncoding(vTokens.at(1)));
+		KIO::TransferJob * pJobForecast = KIO::get( urlForecast, KIO::Reload, KIO::HideProgressInfo );
+		if( pJobForecast )
+		{
+			iJobCounter += 1;
+			pJobForecast->addMetaData("cookies", "none"); // Disable displaying cookies
+			pJobForecast->setObjectName(QString("%1|%2").arg(XmlDataForecast).arg(sServiceId));
+			
+			struct XmlServiceData * pXmlData = new XmlServiceData;
+			pXmlData->sLocation = sLocation;
+			pXmlData->sServiceId = sServiceId;
+			pXmlData->sSource = sSource;
+			pXmlData->pJob = pJobForecast;
+			
+			d->vServiceJobs.insert( pJobForecast->objectName(), pXmlData );
 
-		connect( pJobForecast, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
-			SLOT(slotDataArrived(KIO::Job *, const QByteArray &)) );
-		connect( pJobForecast, SIGNAL(result(KJob *)), this, SLOT(slotJobFinished(KJob *)));
+			connect( pJobForecast, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
+				SLOT(slotDataArrived(KIO::Job *, const QByteArray &)) );
+			connect( pJobForecast, SIGNAL(result(KJob *)), this, SLOT(slotJobFinished(KJob *)));
+		}
 	}
-
-	/*** download alerts ***/
 	
+	/*** download alerts ***/
+	// Not done yet!!!
 	
 	if( iJobCounter >= 1 )
 	{
 		XmlWeatherData * pXmlWeatherData = new XmlWeatherData;
+		pXmlWeatherData->sSource = sSource;
+		pXmlWeatherData->sLocation = sLocation;
 		pXmlWeatherData->iActiveJobs = iJobCounter;
-		pXmlWeatherData->iSuccededJobs = 0;
 		pXmlWeatherData->measureSystem = KGlobal::locale()->measureSystem();
 		
 		d->vWeatherData.insert(sServiceId, pXmlWeatherData);
@@ -678,6 +856,45 @@ WundergroundIon::readCurrentObservation( QXmlStreamReader & xml, XmlWeatherData
 			{
 				if( xml.name().compare("observation_time_rfc822") == 0 )
 					data.sObservationTime = xml.readElementText();
+				else if( xml.name().compare("forecast_url") == 0 )
+					data.sForecastUrl = xml.readElementText();
+				
+				// we try to extract the longitude and latitude for the weather image,
+				// the longitude & latitude does not match the position information
+				// we receive in tag <observation_location>
+				else if( xml.name().compare("ob_url") == 0 )
+				{
+					QString sTmp = xml.readElementText();
+					dInfo() << "Latitude/Longitude" << sTmp;
+					int pos = sTmp.indexOf("?query=");
+					sTmp = sTmp.right(sTmp.length() - pos - 7); // we do not want the text "?query=" in text
+					QStringList vPosition = sTmp.split(',', QString::SkipEmptyParts);
+					dInfo() << "Latitude/Longitude for image" << vPosition;
+					
+					if (vPosition.length() == 2)
+					{
+						QUrl urlSatelliteMap("http://wublast.wunderground.com/cgi-bin/WUBLAST");
+						urlSatelliteMap.addEncodedQueryItem("lat", vPosition[0].toLatin1());
+						urlSatelliteMap.addEncodedQueryItem("lon", vPosition[1].toLatin1());
+						urlSatelliteMap.addEncodedQueryItem("zoom", "3");
+						urlSatelliteMap.addEncodedQueryItem("width", "310");
+						urlSatelliteMap.addEncodedQueryItem("height", "233");
+						urlSatelliteMap.addEncodedQueryItem("key", "sat_ir4_thumb");
+						urlSatelliteMap.addEncodedQueryItem("basemap", "1");
+						urlSatelliteMap.addEncodedQueryItem("gtt", "0");
+						urlSatelliteMap.addEncodedQueryItem("num", "1");
+						urlSatelliteMap.addEncodedQueryItem("timelabel", "0");
+						urlSatelliteMap.addEncodedQueryItem("delay", "25");
+						urlSatelliteMap.addEncodedQueryItem("borders", "1");
+						urlSatelliteMap.addEncodedQueryItem("theme", "WUNIDS");
+						urlSatelliteMap.addEncodedQueryItem("extension", "png");
+
+						dInfo() << "Satellite Map" << urlSatelliteMap;
+						data.imageUrl = urlSatelliteMap;
+						
+						connectWithImageData(urlSatelliteMap);
+					}
+				}
 				else if( xml.name().compare("relative_humidity") == 0 )
 				{
 					data.sHumidity = xml.readElementText().simplified();
@@ -738,6 +955,48 @@ WundergroundIon::readCurrentObservation( QXmlStreamReader & xml, XmlWeatherData
 	dEndFunct();
 }
 
+void
+WundergroundIon::connectWithImageData(const QUrl & urlSatelliteMap)
+{
+	dStartFunct();
+	
+	if (urlSatelliteMap.isEmpty())
+	{
+		dWarning() << "Url for satellite map is empty";
+		dEndFunct();
+	}
+	
+	ImageData * pImageData = 0;
+	if (!d->vImageData.contains(urlSatelliteMap) )
+	{
+		KIO::TransferJob * pJob = KIO::get(urlSatelliteMap, KIO::Reload, KIO::HideProgressInfo);
+		if (pJob)
+		{
+//			pJob->addMetaData("cookies", "none"); // Disable displaying cookies
+			pImageData = new ImageData;
+			pImageData->imageUrl = urlSatelliteMap;
+			pImageData->bReady = false;
+			pImageData->locationCounter = 1;
+			
+			d->vImageJobs.insert( pJob, pImageData );
+			d->vImageData.insert( urlSatelliteMap, pImageData );
+
+			connect( pJob, SIGNAL(data(KIO::Job *, const QByteArray &)),
+			         this, SLOT(slotImageDataArrived(KIO::Job *, const QByteArray &)) );
+			connect(  pJob, SIGNAL(result(KJob *)), this, SLOT(slotImageJobFinished(KJob *)) );
+		}
+	}
+	else
+	{
+		pImageData = d->vImageData[urlSatelliteMap];
+		pImageData->locationCounter += 1;
+	}
+	dDebug() << "Image Jobs: " << d->vImageJobs.count();
+	dDebug() << "Image Data: " << d->vImageData.count();
+	
+	dEndFunct();
+}
+
 void
 WundergroundIon::readWeatherForecast( QXmlStreamReader & xml, XmlWeatherData & data )
 {
@@ -802,20 +1061,27 @@ WundergroundIon::readWeatherForecast( QXmlStreamReader & xml, XmlWeatherData & d
 }
 
 void
-WundergroundIon::updateWeatherSource( const XmlWeatherData & data, const QString & sSource, const QString & sLocation )
+WundergroundIon::updateWeatherSource( const XmlWeatherData & data, const ImageData * pImageData )
 {
-	dStartFunct()	<< sSource;
-	removeAllData( sSource );	// clear the old values
-	setData(sSource, Data());	// start the update timer
+	dStartFunct()	<< data.sSource;
+	removeAllData( data.sSource );	// clear the old values
+	setData(data.sSource, Data());	// start the update timer
 
-	setData(sSource, "Credit", i18n("Supported by Weather Underground Weather Station"));
-	setData(sSource, "Credit Url", "http://wunderground.com");
-	setData(sSource, "Place", sLocation);
+	setData(data.sSource, "Credit", i18n("Supported by Weather Underground NOAA Weather Station"));
+	
+	if (!data.sForecastUrl.isEmpty())
+		setData(data.sSource, "Credit Url", data.sForecastUrl);
+	
+	if( pImageData && !pImageData->image.isNull() )
+		setData(data.sSource, "Satellite Map", pImageData->image);
+
+	
+	setData(data.sSource, "Place", data.sLocation);
 
 	if( !data.sLongitude.isEmpty() && !data.sLatitude.isEmpty() )
 	{
-		setData(sSource, "Longitude", data.sLongitude);
-		setData(sSource, "Latitude", data.sLatitude);
+		setData(data.sSource, "Longitude", data.sLongitude);
+		setData(data.sSource, "Latitude", data.sLatitude);
 	}
 
 	KDateTime obsDate;
@@ -828,7 +1094,7 @@ WundergroundIon::updateWeatherSource( const XmlWeatherData & data, const QString
 		if( obsDate.isValid() && !data.sTimeZone.isEmpty() )
 		{
 			obsDate = obsDate.toZone( KSystemTimeZones::zone(data.sTimeZone) );
-			setData(sSource, "Observation Period", obsDate.toString("%d.%m.%Y @ %k:%M"));
+			setData(data.sSource, "Observation Period", obsDate.toString("%d.%m.%Y @ %k:%M"));
 		}
 	}
 
@@ -867,46 +1133,53 @@ WundergroundIon::updateWeatherSource( const XmlWeatherData & data, const QString
 	#endif
 #endif
 
-	setData(sSource, "Dewpoint Unit", QString::number(iTempSystem) );
-	setData(sSource, "Temperature Unit", QString::number(iTempSystem) );
-	setData(sSource, "Wind Gust Unit", QString::number(iSpeedSystem) );
-	setData(sSource, "Visibility Unit", QString::number(iDistanceSystem) );
-	setData(sSource, "Wind Speed Unit",  QString::number(iSpeedSystem) );
+	setData(data.sSource, "Dewpoint Unit", QString::number(iTempSystem) );
+	setData(data.sSource, "Temperature Unit", QString::number(iTempSystem) );
+	setData(data.sSource, "Wind Gust Unit", QString::number(iSpeedSystem) );
+	setData(data.sSource, "Visibility Unit", QString::number(iDistanceSystem) );
+	setData(data.sSource, "Wind Speed Unit",  QString::number(iSpeedSystem) );
 
-	setData(sSource, "Condition Icon", getWeatherIcon(d->vIconConditionMap, d->stringConverter(data.sCurrentIcon) ) );
-	setData(sSource, "Current Conditions", d->stringConverter(data.sDayCondition) );
+	setData(data.sSource, "Condition Icon", getWeatherIcon(d->vIconConditionMap, d->stringConverter(data.sCurrentIcon) ) );
+	setData(data.sSource, "Current Conditions", d->stringConverter(data.sDayCondition) );
 
-	setData(sSource, "Dewpoint", d->stringConverter(data.sDewpoint) );
-	setData(sSource, "Humidity", d->stringConverter(data.sHumidity) );
-	setData(sSource, "Pressure", d->stringConverter(data.sPressure) );
-	setData(sSource, "Temperature", d->stringConverter(data.sCurrentTemp) );
-	setData(sSource, "Visibility", d->stringConverter(data.sVisibility) );
-	setData(sSource, "Wind Direction", d->stringConverter(data.sWindDirection) );
-	setData(sSource, "Wind Speed", d->stringConverter(data.sWindSpeed) );
-	setData(sSource, "Wind Gust", d->stringConverter(data.sWindGust) );
+	setData(data.sSource, "Dewpoint", d->stringConverter(data.sDewpoint) );
+	setData(data.sSource, "Humidity", d->stringConverter(data.sHumidity) );
+	setData(data.sSource, "Pressure", d->stringConverter(data.sPressure) );
+	setData(data.sSource, "Temperature", d->stringConverter(data.sCurrentTemp) );
+	setData(data.sSource, "Visibility", d->stringConverter(data.sVisibility) );
+	setData(data.sSource, "Wind Direction", d->stringConverter(data.sWindDirection) );
+	setData(data.sSource, "Wind Speed", d->stringConverter(data.sWindSpeed) );
+	setData(data.sSource, "Wind Gust", d->stringConverter(data.sWindGust) );
 	
 	if(data.sPressure.isEmpty() || data.sPressure.compare("NA")!=0)
-		setData(sSource, "Pressure Unit", QString::number(iPressureSystem) );
+		setData(data.sSource, "Pressure Unit", QString::number(iPressureSystem) );
 	
 	if( data.sunrise.isValid() && data.sunset.isValid() )
 	{
 #if KDE_VERSION_MINOR <= 2
-		setData(sSource, "Sunrise At", data.sunrise.toString("h:m") );
-		setData(sSource, "Sunset At",  data.sunset.toString("h:m") );
+		setData(data.sSource, "Sunrise At", data.sunrise.toString("h:m") );
+		setData(data.sSource, "Sunset At",  data.sunset.toString("h:m") );
 #else
 		if( obsDate.isValid() )
 		{
-			KDateTime dateTime(obsDate);
-			dateTime.setTime(data.sunrise);
-			setData(sSource, "Sunrise At", dateTime.toString("%A %B %e, %Y at %H:%M %Z"));
+			// Format of sunrise and sunset: "Tuesday November 29, 2011 at 07:43"
+			QDateTime dateTime(obsDate.dateTime().date(), data.sunrise);
+			setData(data.sSource, "Sunrise At", QString("%1, %2 at %3")
+				.arg(dateTime.toString("dddd MMMM dd"))
+				.arg(dateTime.date().year())
+				.arg(dateTime.toString("HH:mm")));
+			
 			dateTime.setTime(data.sunset);
-			setData(sSource, "Sunset At",  dateTime.toString("%A %B %e, %Y at %H:%M %Z"));
+			setData(data.sSource, "Sunset At", QString("%1, %2 at %3")
+				.arg(dateTime.toString("dddd MMMM dd"))
+				.arg(dateTime.date().year())
+				.arg(dateTime.toString("HH:mm")));
 		}
 #endif
 	}
 	if( data.vForecasts.count() > 0 )
 	{
-		setData(sSource, "Total Weather Days", QString::number(data.vForecasts.count()) );
+		setData(data.sSource, "Total Weather Days", QString::number(data.vForecasts.count()) );
 		
 		QList<XmlForecastDay *>::const_iterator itDay = data.vForecasts.constBegin();
 		short iDay(0);
@@ -921,7 +1194,7 @@ WundergroundIon::updateWeatherSource( const XmlWeatherData & data, const QString
 				else
 					psCurrCondition = (data.sNightCondition.isEmpty() ? NULL : &data.sNightCondition);
 			}
-			setData( sSource,
+			setData( data.sSource,
 					 QString("Short Forecast Day %1").arg(iDay),
 					QString("%1|%2|%3|%4|%5|N/A")
 						.arg( (*itDay)->sDayName )
@@ -942,8 +1215,6 @@ WundergroundIon::Private::setup_readStation( const QString & sLocation,
                                              XmlLookupResult * pLookupResult )
 
 {
-//	D_ASSERT( xml.name().compare("station") == 0, "XML", "Wrong XML-tag to parse!");
-	
 	QString sCity;
 	QString sNeighborhood;
 	QString sState;
@@ -988,20 +1259,29 @@ WundergroundIon::Private::setup_readStation( const QString & sLocation,
 				break;
 		}
 	}
-	dDebug() << (stationType == AirportStation ? "Airport" : "PWS") << sCity << sNeighborhood << sState << sCountry;
+	dDebug() << (stationType == AirportStation ? "Airport" : "PWS") << sCity << sNeighborhood << sState << sCountry << "ID =" << sId;
 
-	if( sCity.isEmpty() || sId.isEmpty() )
+	if (sCity.isEmpty() ||
+	    sId.isEmpty() ||
+	    sId.compare("----") == 0) // station ID is invalid, weather provider will not return any values for station ID "----"
 		return;
-	if( !sNeighborhood.isEmpty() )
+
+	if (!sNeighborhood.isEmpty())
 		sCity.append(" - " + sNeighborhood);
 	
 	QString sKey;
 	QString sServiceId;
 	QString sLocationExtras;
-	if( sState.isEmpty() )
-		sKey = QString("%1|%2").arg(sCountry).arg(sCity);
+	
+	if (stationType == AirportStation)
+		sKey = "Airport";
 	else
-		sKey = QString("%1|%2|%3").arg(sCountry).arg(sState).arg(sCity);
+		sKey = "PWS";
+	
+	if( sState.isEmpty() )
+		sKey += QString("%1|%2").arg(sCity).arg(sCountry);
+	else
+		sKey += QString("%1|%2|%3").arg(sCity).arg(sState).arg(sCountry);
 
 	if( stationType == AirportStation )
 	{
@@ -1010,11 +1290,16 @@ WundergroundIon::Private::setup_readStation( const QString & sLocation,
 	}
 	else
 	{
+		sLocationExtras += "stationtype|PWS";
 		sServiceId = QString("pws:%1").arg(sId);
 	}
 	if( !sDistance.isEmpty() )
+	{
+		if (!sLocationExtras.isEmpty())
+			sLocationExtras.append("|");
+		
 		sLocationExtras += "distance|"+sDistance;
-	
+	}
 	if( sCountry.compare("US") == 0 )
 	{
 		sCountry = sState;
@@ -1026,13 +1311,15 @@ WundergroundIon::Private::setup_readStation( const QString & sLocation,
 	QString sData( QString("place|%1|extra|%2").arg(sLocationName).arg(sServiceId) );
 
 //	dDebug() << sKey << sServiceId << sData << sLocationExtras;
-	if( pLookupResult && !pLookupResult->vLocationExtras.contains(sServiceId) )
+	if( pLookupResult &&
+	    sLocationExtras.length() > 0 &&
+	    !pLookupResult->vLocationExtras.contains(sServiceId) )
 	{
 		pLookupResult->vCities.insert(sKey, sData);
 		pLookupResult->vLocationExtras.insert(sServiceId, sLocationExtras);
 	}
 	else
-		dWarning() << "Can not add Station \"" << sData << "\" to source" << sLocation;
+		dWarning() << "StationID already exists:" << sData << "with" << sLocation;
 
 }
 
@@ -1041,8 +1328,6 @@ WundergroundIon::Private::parseForecastDay( QXmlStreamReader & xml,
                                             const KLocale::MeasureSystem system,
                                             QString & sTimeZone ) const
 {
-//	D_ASSERT( xml.name().compare("forecastday") == 0, "XML", "Wrong XML-tag to parse!");
-
 	XmlForecastDay * pDay = new XmlForecastDay;
 	
 	short iLevel = 1;	// we have parsed the start tag already, and decided to call this function!!!
@@ -1109,8 +1394,6 @@ WundergroundIon::Private::parseForecastTemp( QXmlStreamReader & xml, const KLoca
 void
 WundergroundIon::Private::parseTextCondition( QXmlStreamReader & xml, XmlWeatherData & data ) const
 {
-//	D_ASSERT( xml.name().compare("forecastday") == 0, "XML", "Wrong XML-tag to parse!");
-	
 	short   iLevel(1);
 	short   iPeriode(-1);
 	QString sText;
