@@ -57,6 +57,7 @@
 
 struct XmlWeatherData;
 struct XmlLookupResult;
+struct ImageData;
 
 class KDE_EXPORT WundergroundIon : public IonInterface
 {
@@ -99,6 +100,9 @@ private slots:
 
 	void slotDataArrived( KIO::Job * job, const QByteArray & data );
 	void slotJobFinished( KJob * job );
+	
+	void slotImageDataArrived( KIO::Job * job, const QByteArray & data );
+	void slotImageJobFinished( KJob * job );
 
 private:
 	void setup_findPlace( const QString & sLocation, const QString & sSource, const QString & sPath = QString() );
@@ -107,7 +111,8 @@ private:
 	bool getWeatherData( const QString & sLocation, const QString & sServiceId, const QString & sSource );
 	void readCurrentObservation( QXmlStreamReader & xml, XmlWeatherData & data );
 	void readWeatherForecast( QXmlStreamReader & xml, XmlWeatherData & data );
-	void updateWeatherSource( const XmlWeatherData & data, const QString & sSource, const QString & sLocation );
+	void connectWithImageData(const QUrl & url);
+	void updateWeatherSource( const XmlWeatherData & data, const ImageData * imageData);
 
 	void cleanup();
 	
