@@ -39,6 +39,7 @@ IonTester::IonTester( QObject * parent )
 	: QObject(parent),
 	  m_ion( this, QVariantList() )
 {
+      m_ion.init();
 }
 IonTester::~IonTester()
 {
@@ -95,17 +96,19 @@ main(int argc, char *argv[])
 
 
 	IonTester tester;
+
 /*	tester.connectSource( "accuweather|weather|Berlin, Germany(Berlin)|EUR.DE.GM003.BERLIN" );
 	tester.reset();
 	tester.connectSource( "accuweather|validate|Dresden" );
 */	tester.connectSource( "accuweather|weather|Milwaukee, NC|27854" );
 
-/*	tester.connectSource( "google|weather|Berlin" );
-	tester.reset();
-	tester.connectSource("google|validate|Praha");
-	tester.connectSource("google|weather|Praha");
+//	tester.connectSource( "google|weather|Berlin" );
+//	tester.reset();
+//	tester.connectSource("google|validate|springfield, ma");
+//	tester.connectSource("google|weather|Springfield, MA");
+//	tester.connectSource("google|weather|Hartford, CT");
 //	dDebug() << WeatherUtils::convert( 5.0, WeatherUtils::Miles, WeatherUtils::Kilometers );
-*/
+
 	QTimer::singleShot(30*1000, &app, SLOT(quit()));
 	return app.exec();
 }
